@@ -3,7 +3,7 @@
 i = 0
 
 timer_period[0] = 250
-leds_cicle = [0, 0, 0, 0, 0, 0, 0, 0]
+leds_cicle = 8 * [0]
 
 @onevent
 def timer0():
