@@ -286,6 +286,54 @@ class Context:
 
         return None
 
+    def numeric_constant(self, node):
+        """If node is a numeric constant (literal number or len(list)), get its
+        value; else return None.
+        """
+        if isinstance(node, ast.Constant) and isinstance(node.value, int):
+            return node.value
+        elif isinstance(node, ast.Num):  # 3.6
+            return node.n
+        elif isinstance(node, ast.Call) and ATranspiler.decode_attr(node.func) == "len":
+            if len(node.args) != 1:
+                return None
+            if isinstance(node.args[0], ast.Name):
+                return self.var_array_size(node.args[0].id)
+            else:
+                return self.list_length(node.args[0])
+
+    def list_length(self, node):
+        """If node is a list, either [...], [...] * cst or cst * [...], get its
+        length; else return None.
+        """
+        if isinstance(node, ast.List):
+            return len(node.elts)
+        elif isinstance(node, ast.BinOp) and isinstance(node.op, ast.Mult):
+            if isinstance(node.left, ast.List):
+                n = self.numeric_constant(node.right)
+                if n is not None:
+                    return len(node.left.elts) * n
+            elif isinstance(node.right, ast.List):
+                n = self.numeric_constant(node.left)
+                if n is not None:
+                    return n * len(node.right.elts)
+
+    def list_elements(self, node):
+        """If node is a list, either [...], [...] * cst or cst * [...], get its
+        elements; else return None.
+        """
+        if isinstance(node, ast.List):
+            return node.elts
+        elif isinstance(node, ast.BinOp) and isinstance(node.op, ast.Mult):
+            if isinstance(node.left, ast.List):
+                n = self.numeric_constant(node.right)
+                if n is not None:
+                    return node.left.elts * n
+            elif isinstance(node.right, ast.List):
+                n = self.numeric_constant(node.left)
+                if n is not None:
+                    return n * node.right.elts
+
 
 class AFunction:
     """Transpilation information for aseba functions.
@@ -322,7 +370,7 @@ class AFunction:
                 # array arg
                 if isinstance(arg, ast.Name):
                     arg_code.append(arg.id)
-                elif atranspiler.list_length(arg) is not None:
+                elif context.list_length(arg) is not None:
                     value, aux_st, _ = atranspiler.compile_expr(arg, context, atranspiler.PRI_ASSIGN)
                     aux_statements += aux_st
                     arg_code.append(value)
@@ -433,40 +481,6 @@ class ATranspiler:
             raise TranspilerError("invalid name", ast_node=node)
         return name
 
-    @staticmethod
-    def list_length(node):
-        """If node is a list, either [...], [...] * cst or cst * [...], get its
-        length; else return None.
-        """
-        if isinstance(node, ast.List):
-            return len(node.elts)
-        elif isinstance(node, ast.BinOp) and isinstance(node.op, ast.Mult):
-            if isinstance(node.left, ast.List) and isinstance(node.right, ast.Constant) and isinstance(node.right.value, int):
-                return len(node.left.elts) * node.right.value
-            elif isinstance(node.left, ast.Constant) and isinstance(node.left.value, int) and isinstance(node.right, ast.List):
-                return node.left.value * len(node.right.elts)
-            elif isinstance(node.left, ast.List) and isinstance(node.right, ast.Num):  # 3.6
-                return len(node.left.elts) * node.right.n
-            elif isinstance(node.left, ast.Num) and isinstance(node.right, ast.List):
-                return node.left.n * len(node.right.elts)
-
-    @staticmethod
-    def list_elements(node):
-        """If node is a list, either [...], [...] * cst or cst * [...], get its
-        elements; else return None.
-        """
-        if isinstance(node, ast.List):
-            return node.elts
-        elif isinstance(node, ast.BinOp) and isinstance(node.op, ast.Mult):
-            if isinstance(node.left, ast.List) and isinstance(node.right, ast.Constant) and isinstance(node.right.value, int):
-                return node.left.elts * node.right.value
-            elif isinstance(node.left, ast.Constant) and isinstance(node.left.value, int) and isinstance(node.right, ast.List):
-                return node.left.value * node.right.elts
-            elif isinstance(node.left, ast.List) and isinstance(node.right, ast.Num):  # 3.6
-                return node.left.elts * node.right.n
-            elif isinstance(node.left, ast.Num) and isinstance(node.right, ast.List):
-                return node.left.n * node.right.elts
-
     def split(self, parent_context):
         """Split the Python source code into top-level source code, which is
         returned, and function definitions, which are stored into parent_context.
@@ -535,10 +549,10 @@ class ATranspiler:
         priority = self.PRI_HIGH
         is_boolean = False
 
-        if self.list_length(node) is not None:
+        if context.list_length(node) is not None:
             if priority_container > self.PRI_ASSIGN:
                 raise TranspilerError("list not supported in expression", node)
-            for el in self.list_elements(node):
+            for el in context.list_elements(node):
                 el_code, aux_st, is_boolean = self.compile_expr(el, context, self.PRI_NUMERIC)
                 aux_statements += aux_st
                 if code is None:
@@ -653,7 +667,7 @@ end
                     len_arg_size = context.var_array_size(node.args[0].id)
                     code = f"{len_arg_size}" if len_arg_size is not None else "0"
                 else:
-                    list_len = self.list_length(node.args[0])
+                    list_len = context.list_length(node.args[0])
                     if list_len is None:
                         raise TranspilerError("type of argument of len is not a list", ast_node=node)
                     code = f"{list_len}"
@@ -835,7 +849,7 @@ end
             target_str = context.var_str(target, True)
             if index is None:
                 # declare target
-                list_len = self.list_length(node.value)
+                list_len = context.list_length(node.value)
                 if list_len is not None:
                     # var = [...]
                     target_size = list_len
@@ -843,7 +857,7 @@ end
                 elif isinstance(node.value, (ast.Name, ast.Attribute)):
                     # var1 = var2: inherit size
                     name_right = self.decode_attr(node.value)
-                    target_size = context.var_array_size(name_right, is_target=True)
+                    target_size = context.var_array_size(name_right, is_target=False)
                     if target_size is False:
                         raise TranspilerError(f"unknown variable '{name_right}'", node)
                     context.declare_var(target, target_size, ast_node=node)
@@ -858,7 +872,7 @@ end
                 else:
                     context.declare_var(target, ast_node=node)
             # compile value
-            if self.list_length(node.value) is not None:
+            if context.list_length(node.value) is not None:
                 if index is not None:
                     raise TranspilerError("list assigned to indexed variable", node)
                 value, aux_statements, is_boolean = self.compile_expr(node.value, context, self.PRI_ASSIGN)
