@@ -162,10 +162,11 @@ class ServerThread(threading.Thread):
                                     node.stack_size,
                                     [
                                         (
-                                            len(node.variables[name]),
+                                            i,
                                             name,
+                                            len(node.variables[name]),
                                         )
-                                        for name in node.variables
+                                        for i, name in enumerate(node.variables)
                                     ],
                                     [],
                                     [],
@@ -248,7 +249,7 @@ class ServerThread(threading.Thread):
                         request_id = FlatBuffer.field_val(fb.root.union_data[0].fields[0], 0)
                         node_id_str = ThymioFB.bytes_to_id_str(fb.root.union_data[0].fields[1])
                         variables = {
-                            v.fields[0][0]: v.fields[1][0]
+                            v.fields[0][0]: v.fields[1][0] if isinstance(v.fields[1][0], list) else [v.fields[1][0]]
                             for v in fb.root.union_data[0].fields[2][0]
                         }
                         node = self.server.findNode(node_id_str)
@@ -284,6 +285,26 @@ class ServerThread(threading.Thread):
                             bp.fields[0][0]
                             for bp in fb.root.union_data[0].fields[2][0]
                         ]
+                        node = self.server.findNode(node_id_str)
+                        if node is not None:
+                            msg = self.thymio.create_message((
+                                ThymioFB.MESSAGE_TYPE_SET_BREAKPOINTS_RESPONSE,
+                                (
+                                    request_id,
+                                    ThymioFB.ERROR_NO_ERROR,
+                                    [
+                                        (
+                                            bp,
+                                        )
+                                        for bp in breakpoints
+                                    ],
+                                )
+                            ), ThymioFB.SCHEMA)
+                            self.send_packet(msg)
+                        else:
+                            msg = self.thymio.create_msg_error(request_id, ThymioFB.ERROR_UNKNOWN_NODE)
+                            self.send_packet(msg)
+                            print(f"-> unknown node {node.id}")
                     elif fb.root.union_type == ThymioFB.MESSAGE_TYPE_SET_VM_EXECUTION_STATE:
                         request_id = FlatBuffer.field_val(fb.root.union_data[0].fields[0], 0)
                         node_id_str = ThymioFB.bytes_to_id_str(fb.root.union_data[0].fields[1])
@@ -292,22 +313,31 @@ class ServerThread(threading.Thread):
                             command = FlatBuffer.field_val(fb.root.union_data[0].fields[2], ThymioFB.VM_EXECUTION_STATE_COMMAND_STOP)
                             if command == ThymioFB.VM_EXECUTION_STATE_COMMAND_STOP:
                                 node.execution_state = ThymioFB.VM_EXECUTION_STATE_STOPPED
+                                print("Set vm execution state: stop")
                             elif command == ThymioFB.VM_EXECUTION_STATE_COMMAND_RUN:
                                 node.execution_state = ThymioFB.VM_EXECUTION_STATE_RUNNING
+                                print("Set vm execution state: run")
                             elif command == ThymioFB.VM_EXECUTION_STATE_COMMAND_STEP:
                                 node.execution_state = ThymioFB.VM_EXECUTION_STATE_PAUSED
+                                print("Set vm execution state: step")
                             elif command == ThymioFB.VM_EXECUTION_STATE_COMMAND_STEP_TO_NEXT_LINE:
                                 node.execution_state = ThymioFB.VM_EXECUTION_STATE_PAUSED
+                                print("Set vm execution state: step to next line")
                             elif command == ThymioFB.VM_EXECUTION_STATE_COMMAND_PAUSE:
                                 node.execution_state = ThymioFB.VM_EXECUTION_STATE_PAUSED
+                                print("Set vm execution state: pause")
                             elif command == ThymioFB.VM_EXECUTION_STATE_COMMAND_RESET:
                                 node.execution_state = ThymioFB.VM_EXECUTION_STATE_RUNNING
+                                print("Set vm execution state: reset")
                             elif command == ThymioFB.VM_EXECUTION_STATE_COMMAND_REBOOT:
                                 node.execution_state = ThymioFB.VM_EXECUTION_STATE_STOPPED
+                                print("Set vm execution state: reboot")
                             elif command == ThymioFB.VM_EXECUTION_STATE_COMMAND_SUSPEND:
                                 node.execution_state = ThymioFB.VM_EXECUTION_STATE_PAUSED
+                                print("Set vm execution state: suspend")
                             elif command == ThymioFB.VM_EXECUTION_STATE_COMMAND_WRITE_PROGRAM_TO_DEVICE_MEMORY:
                                 node.execution_state = ThymioFB.VM_EXECUTION_STATE_STOPPED
+                                print("Set vm execution state: write program to device memory")
                             msg = self.thymio.create_msg_request_completed(request_id)
                             self.send_packet(msg)
                             print(f"-> ok")
